@@ -113,11 +113,7 @@ int Xcurl(PGLOBAL g, PCSZ Http, PCSZ Uri, PCSZ filename)
 
 	} // endif f
 	
-#ifdef HAVE_VFORK
-       pID = vfork();
-#else
-       pID = fork();
-#endif
+	pID = vfork();
 	sprintf(fn, "-o%s", filename);
 
 	if (pID == 0) {
